/**
   A simple implementation of the
   IntList abstract data type
   using an int array.
*/
public class IntArrayList implements IntList
{
   private int[] data;
   private int manyItems;

   /**
      Construct an empty array list with an intial capacity of 10.
   */
   public IntArrayList()
   {
      final int INITIAL_CAPACITY = 10;
      manyItems = 0;
      data = new int[INITIAL_CAPACITY];
   }

   /**
      Construct an empty array list with the given initial capacity.

      @param initialCapacity the initial capacity of this IntArrayList
   */
   public IntArrayList(int initialCapacity)
   {
      if (initialCapacity < 0)
         throw new IllegalArgumentException
         ("The initialCapacity is negative: " + initialCapacity);
      data = new int[initialCapacity];
      manyItems = 0;
   }

   /**
      Change the current capacity of this IntArrayList.

      @param minimumCapacity the new capacity for this IntArrayList
   */
   public void ensureCapacity(int minimumCapacity)
   {
      int[] biggerArray;

      if (data.length < minimumCapacity)
      {
         biggerArray = new int[minimumCapacity];
         System.arraycopy(data, 0, biggerArray, 0, manyItems);
         data = biggerArray;
      }
   }

   /**
      Accessor method to get the current capacity of this IntArrayList.

      @return the current capacity of this IntArrayList
   */
   public int getCapacity( )
   {
      return data.length;
   }

   /**
      Reduce the current capacity of this IntArrayList to its
      actual size (i.e., the number of elements it contains).
   */
   public void trimToSize( )
   {
      int[] trimmedArray;

      if (data.length != manyItems)
      {
         trimmedArray = new int[manyItems];
         System.arraycopy(data, 0, trimmedArray, 0, manyItems);
         data = trimmedArray;
      }
   }


   /**
   * {@inheritDoc}
   */
   public int length()
   {
      return manyItems;
   }

   /**
   * {@inheritDoc}
   */
   public boolean empty()
   {
      return (0 == manyItems);
   }

   /**
   * {@inheritDoc}
   */
   public int getFirst()
   {
      return data[manyItems - 1];
   }

   /**
   * {@inheritDoc}
   */
   public void insert(int element)
   {
      if (manyItems == data.length)
      {
         // Ensure twice as much space as we need.
         ensureCapacity( 2 * (manyItems + 1) );
      }

      this.data[manyItems] = element;
      manyItems++;
   }

   /**
   * {@inheritDoc}
   */
   public int remove()
   {
      manyItems--;
      return data[manyItems];
   }

   /**
   * {@inheritDoc}
   */
   public int[] toArray()
   {
      int[] result = new int[manyItems];
      for (int i = 0; i < manyItems; i++)
      {
         result[i] = data[i];
      }
      return result;
   }

   /**
   * {@inheritDoc}
   */
   public String toString()
   {
      String result = "[";
      for (int i = 0; i < manyItems; i++)
      {
         if (i > 0)
         {
            result += " ";
         }
         result += data[i];
         if (i < manyItems-1)
         {
            result += ",";
         }
      }
      result += "]";
      return result;
   }

}//IntArrayList
