/**
   A simple implementation of the
   IntList abstract data type
   as a linked list.
*/
public class IntLinkedList implements IntList
{
   private IntNode first;

   /**
      Construct an empty linked list.
   */
   public IntLinkedList()
   {
      first = null;
   }

   /**
   * {@inheritDoc}
   */
   public int length()
   {
      IntNode cursor = first;
      int length = 0;

      while (cursor != null)
      {
         length++;
         cursor = cursor.getNext();
      }
      return length;
   }

   /**
   * {@inheritDoc}
   */
   public boolean empty()
   {
      return (first == null);
   }

   /**
   * {@inheritDoc}
   */
   public int getFirst()
   {
      return first.getData();
   }

   /**
   * {@inheritDoc}
   */
   public void insert(int element)
   {
      IntNode node = new IntNode(element, first);
      first = node;
   }

   /**
   * {@inheritDoc}
   */
   public int remove()
   {
      int data = first.getData();
      first = first.getNext();
      return data;
   }

   /**
   * {@inheritDoc}
   */
   public int[] toArray()
   {
      int[] result = new int[this.length()];

      int index = 0;
      IntNode cursor = first;
      while ( cursor != null )
      {
         result[index++] = cursor.getData();
         cursor = cursor.getNext();
      }

      return result;
   }

   /**
   * {@inheritDoc}
   */
   public String toString()
   {
      String result = "[ ";
      IntNode cursor = first;
      while ( cursor != null )
      {
         result += cursor.getData() + " ";
         cursor = cursor.getNext();
      }
      result += "]";
      return result;
   }

}//IntLinkedList
